package com.winhc.max.compute.graph.job.enterprise_group;

import com.aliyun.odps.graph.Edge;
import com.aliyun.odps.io.Text;
import com.aliyun.odps.io.Tuple;
import com.aliyun.odps.utils.StringUtils;
import com.winhc.max.compute.graph.job.enterprise_group.entity.HolderEdge;
import com.winhc.max.compute.graph.util.WinhcTuple;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author: XuJiakai
 * 2023/2/8 10:30
 */
public class EnterpriseGroupUtils {


    /**
     * 获取大股东
     * 执行事务合伙人要绝对控股
     *
     * @param edges 所有股东
     * @return
     */
    public static Map<String, Double> getMajorityShareholder(List<Edge<Text, HolderEdge>> edges) {
        return getMajorityShareholder(edges, null);
    }

    public static Map<String, Double> getMajorityShareholder(List<Edge<Text, HolderEdge>> edges, List<String> legalEntityIds) {
        Map<String, Double> map = new HashMap<>();

        if (legalEntityIds != null && !legalEntityIds.isEmpty()) {
            Set<String> set = new HashSet<>(legalEntityIds);
            return edges.stream().filter(e -> set.contains(e.getDestVertexId().toString())).collect(Collectors.toMap(e -> e.getDestVertexId().toString(), e -> e.getValue().getInvestmentProportion().get(), (e1, e2) -> e1));
        }

        double holdProportion = 0.0;

        for (Edge<Text, HolderEdge> edge : edges) {
            String destVertexId = edge.getDestVertexId().toString();

            HolderEdge value = edge.getValue();
            double v = value.getInvestmentProportion().get();
            int compare = Double.compare(v, holdProportion);
            if (compare > 0) {
                map.clear();
                map.put(destVertexId, v);
                holdProportion = v;
            } else if (compare == 0 && v > 0) {
                map.put(destVertexId, v);
            }
        }
        return map;
    }


    /**
     * 获取非控股的其它股东id
     *
     * @param edges                    所有股东
     * @param majorityShareholderKeyno 控股股东id
     * @return
     */
    public static List<String> getOtherHolder(List<Edge<Text, HolderEdge>> edges, List<String> majorityShareholderKeyno) {
        Set<String> set = new HashSet<>(majorityShareholderKeyno);

        List<String> collect = edges.stream().map(Edge::getDestVertexId).map(Text::toString).filter(e -> !set.contains(e)).collect(Collectors.toList());
        return collect;
    }


    public static List<String> tuple2List(Tuple tuple) {
        if (tuple == null) {
            return Collections.emptyList();
        }


        List<String> collect = tuple.getAll().stream().filter(Objects::nonNull).map(e -> ((Text) e).toString()).collect(Collectors.toList());
        return collect;
    }

    public static Tuple list2Tuple(List<String> list) {
        if (list == null) {
            return new Tuple();
        }
        Tuple tuple = new Tuple(list.size());
        for (String ele : list) {
            tuple.append(new Text(ele));
        }
        return tuple;
    }


    private static WinhcTuple<String, String> readNode(String chain) {
        int startIndex = chain.indexOf("(") + 1;
        int endIndex = chain.indexOf(")");
        String nodeId = chain.substring(startIndex, endIndex);
        return WinhcTuple.of(nodeId, chain.substring(endIndex + 1));
    }

    private static WinhcTuple<String, String> readEdge(String chain) {
        int startIndex = chain.indexOf("[") + 1;
        int endIndex = chain.indexOf("]");
        String edge = chain.substring(startIndex, endIndex);
        return WinhcTuple.of(edge, chain.substring(endIndex + 1));
    }


    public static Set<String> splitStockChain(String stockChain) {
        if (StringUtils.isBlank(stockChain)) {
            return new HashSet<>();
        }
        Set<String> result = new HashSet<>();

        for (String chain : stockChain.split(",")) {
            String tempNodeId = null;

            while (chain.length() > 10) {
                String node1;
                if (tempNodeId == null) {
                    WinhcTuple<String, String> tuple = readNode(chain);
                    node1 = tuple._1;
                    chain = tuple._2;
                } else {
                    node1 = tempNodeId;
                }
                WinhcTuple<String, String> tuple = readEdge(chain);
                String edge = tuple._1;
                chain = tuple._2;
                tuple = readNode(chain);
                String node2 = tuple._1;
                chain = tuple._2;
                result.add("(" + node1 + ")<-[" + edge + "]-(" + node2 + ")");
                tempNodeId = node2;
            }
        }
        return result;
    }


    private static Set<String> getRelationNodeId(String rootNodeId, Set<String> chains) {
        return chains.stream()
                .filter(e -> e.split("]")[1].contains(rootNodeId))
                .map(e -> readNode(e)._1)
                .collect(Collectors.toSet());
    }


    private static void getChildNode(Set<String> resultNode, Set<String> chains, int nodeLimitNum, String rootId, int leveNum) {
        if (resultNode.size() >= nodeLimitNum) {
            return;
        }

        Set<String> thisNodeSet = getRelationNodeId(rootId, chains);
        String[] thisNodeArray = thisNodeSet.toArray(new String[0]);

        if (resultNode.size() + thisNodeSet.size() >= nodeLimitNum) {
            int t = nodeLimitNum - resultNode.size();
            for (int i = 0; i < t; i++) {
                resultNode.add(thisNodeArray[i]);
            }
            return;
        } else {
            resultNode.addAll(thisNodeSet);
            for (String s : thisNodeArray) {
                getChildNode(resultNode, chains, nodeLimitNum, s, leveNum + 1);
            }
        }
    }


    /**
     * @param rootNodeId
     * @param stockChain
     * @param nodeLimitNum
     * @return
     */
    public static String stockChainTrim(String rootNodeId, String stockChain, int nodeLimitNum) {
        Set<String> nodeIdSet = new HashSet<>();
        Map<Integer, Set<String>> map = new HashMap<>();
        Set<String> chain = splitStockChain(stockChain);

        Set<String> tmpSet = new HashSet<String>() {{
            add(rootNodeId);
        }};
        map.put(0, tmpSet);
        nodeIdSet.addAll(tmpSet);

        getChildNode(nodeIdSet, chain, nodeLimitNum, rootNodeId, 1);

        Stream<String> stringStream = chain.stream().filter(e -> {
            WinhcTuple<String, String> tuple = readNode(e);
            WinhcTuple<String, String> tuple1 = readNode(tuple._2);
            return nodeIdSet.contains(tuple1._1) && nodeIdSet.contains(tuple._1);
        });
        return stringStream.collect(Collectors.joining(","));
    }

    public static void main(String[] args) {
        System.out.println(stockChainTrim("b", "(a)<-[1]-(b),(c)<-[1]-(b),(d)<-[1]-(b),(c)<-[1]-(d)", 3));
    }
}
